'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isModified = exports.isSavePending = exports.isReadPending = exports.getValidationErrors = exports.getNewSubrecordCsid = exports.getNewData = exports.getRelationUpdatedTimestamp = exports.getSubrecordCsid = exports.getError = exports.getData = undefined;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _recordDataHelpers = require('../helpers/recordDataHelpers');

var _configHelpers = require('../helpers/configHelpers');

var _record = require('../actions/record');

var _relation = require('../actions/relation');

var _idGenerator = require('../actions/idGenerator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var newRecordCsid = '';

var getCurrentData = function getCurrentData(state, csid) {
  return state.getIn([csid, 'data', 'current']);
};
var setCurrentData = function setCurrentData(state, csid, data) {
  return state.setIn([csid, 'data', 'current'], data);
};

var getBaselineData = function getBaselineData(state, csid) {
  return state.getIn([csid, 'data', 'baseline']);
};
var setBaselineData = function setBaselineData(state, csid, data) {
  return state.setIn([csid, 'data', 'baseline'], data);
};

var addFieldInstance = function addFieldInstance(state, action) {
  var _action$meta = action.meta,
      csid = _action$meta.csid,
      path = _action$meta.path,
      recordTypeConfig = _action$meta.recordTypeConfig;


  var data = getCurrentData(state, csid);

  if (!data) {
    return state;
  }

  var value = (0, _recordDataHelpers.deepGet)(data, path);
  var list = _immutable2.default.List.isList(value) ? value : _immutable2.default.List.of(value);

  var fieldDescriptor = (0, _get2.default)(recordTypeConfig, ['fields'].concat(_toConsumableArray((0, _configHelpers.dataPathToFieldDescriptorPath)(path))));
  var defaultData = (0, _recordDataHelpers.applyDefaults)(fieldDescriptor);

  var updatedData = (0, _recordDataHelpers.deepSet)(data, path, list.push(defaultData));

  return setCurrentData(state, csid, updatedData);
};

var doCreateNew = function doCreateNew(state, config, recordTypeConfig, cloneCsid, subrecordName) {
  var data = void 0;

  if (cloneCsid) {
    data = (0, _recordDataHelpers.cloneRecordData)(recordTypeConfig, getCurrentData(state, cloneCsid));
  }

  if (!data) {
    data = (0, _recordDataHelpers.createRecordData)(recordTypeConfig);
  }

  var csid = subrecordName ? newRecordCsid + '/' + subrecordName : newRecordCsid;

  var updatedState = state;

  updatedState = setBaselineData(updatedState, csid, data);
  updatedState = setCurrentData(updatedState, csid, data);

  var subrecords = recordTypeConfig.subrecords;


  if (subrecords) {
    Object.keys(subrecords).forEach(function (name) {
      var subrecordConfig = subrecords[name];
      var subrecordType = subrecordConfig.recordType;
      var subrecordTypeConfig = (0, _get2.default)(config, ['recordTypes', subrecordType]);
      var subrecordCsid = state.getIn([cloneCsid, 'subrecord', name]);

      updatedState = doCreateNew(updatedState, config, subrecordTypeConfig, subrecordCsid, name);

      updatedState = updatedState.setIn([csid, 'subrecord', name], csid + '/' + name);
    });
  }

  return updatedState;
};

var createNewRecord = function createNewRecord(state, action) {
  var _action$meta2 = action.meta,
      config = _action$meta2.config,
      recordTypeConfig = _action$meta2.recordTypeConfig,
      cloneCsid = _action$meta2.cloneCsid;


  return doCreateNew(state, config, recordTypeConfig, cloneCsid);
};

var deleteFieldValue = function deleteFieldValue(state, action) {
  var _action$meta3 = action.meta,
      csid = _action$meta3.csid,
      path = _action$meta3.path;


  var data = getCurrentData(state, csid);

  if (!data) {
    return state;
  }

  var updatedData = (0, _recordDataHelpers.deepDelete)(data, path);

  return setCurrentData(state, csid, updatedData);
};

var moveFieldValue = function moveFieldValue(state, action) {
  var _action$meta4 = action.meta,
      csid = _action$meta4.csid,
      path = _action$meta4.path,
      newPosition = _action$meta4.newPosition;


  var data = getCurrentData(state, csid);

  if (!data) {
    return state;
  }

  var listPath = path.slice(0, -1);
  var oldPosition = path[path.length - 1];

  var list = (0, _recordDataHelpers.deepGet)(data, listPath);

  if (!_immutable2.default.List.isList(list)) {
    return state;
  }

  var value = list.get(oldPosition);

  list = list.delete(oldPosition);
  list = list.insert(newPosition, value);

  var updatedData = (0, _recordDataHelpers.deepSet)(data, listPath, list);

  return setCurrentData(state, csid, updatedData);
};

var setFieldValue = function setFieldValue(state, action) {
  var _action$meta5 = action.meta,
      csid = _action$meta5.csid,
      path = _action$meta5.path;


  var data = getCurrentData(state, csid);

  if (!data) {
    return state;
  }

  var newValue = action.payload;
  var updatedData = (0, _recordDataHelpers.deepSet)(data, path, newValue);
  var updatedState = setCurrentData(state, csid, updatedData);

  return updatedState;
};

var handleRecordReadFulfilled = function handleRecordReadFulfilled(state, action) {
  var data = _immutable2.default.fromJS(action.payload.data);

  var csid = action.meta.csid;


  var updatedState = state.deleteIn([csid, 'isReadPending']);

  updatedState = setBaselineData(updatedState, csid, data);
  updatedState = setCurrentData(updatedState, csid, data);

  return updatedState;
};

var handleRecordSaveFulfilled = function handleRecordSaveFulfilled(state, action) {
  var data = _immutable2.default.fromJS(action.payload.data);

  var _action$meta6 = action.meta,
      csid = _action$meta6.csid,
      relatedSubjectCsid = _action$meta6.relatedSubjectCsid;


  var updatedState = state;

  updatedState = updatedState.deleteIn([csid, 'isSavePending']);

  updatedState = setBaselineData(updatedState, csid, data);
  updatedState = setCurrentData(updatedState, csid, data);

  if (relatedSubjectCsid) {
    updatedState = updatedState.setIn([relatedSubjectCsid, 'relationUpdatedTime'], (0, _recordDataHelpers.getUpdatedTimestamp)(data));
  }

  return updatedState;
};

var revertRecord = function revertRecord(state, action) {
  var csid = action.meta.csid;


  var updatedState = setCurrentData(state, csid, getBaselineData(state, csid));

  // Revert subrecords.

  var subrecords = state.getIn([csid, 'subrecord']);

  if (subrecords) {
    subrecords.forEach(function (subrecordCsid) {
      updatedState = revertRecord(updatedState, {
        meta: {
          csid: subrecordCsid
        }
      });
    });
  }

  return updatedState;
};

var handleSubjectRelationsUpdated = function handleSubjectRelationsUpdated(state, action) {
  // Currently relations are only ever created (not updated), and we don't bother to retrieve
  // the relation record after creation. Technically we should retrieve the new relation
  // record and use its updatedAt value here, but that's an extra request. For now just use the
  // current local time, at least until there's some additional reason to retrieve the full
  // relation record.

  var subjectCsid = action.meta.csid;
  var newUpdatedTime = new Date().toISOString();

  if (state.has(subjectCsid)) {
    return state.setIn([subjectCsid, 'relationUpdatedTime'], newUpdatedTime);
  }

  return state;
};

var handleCreateIDFulfilled = function handleCreateIDFulfilled(state, action) {
  var _action$meta7 = action.meta,
      csid = _action$meta7.csid,
      path = _action$meta7.path;


  var data = getCurrentData(state, csid);

  if (!data) {
    return state;
  }

  var newValue = action.payload.data;
  var updatedData = (0, _recordDataHelpers.deepSet)(data, path, newValue);
  var updatedState = setCurrentData(state, csid, updatedData);

  return updatedState;
};

var handleValidationFailed = function handleValidationFailed(state, action) {
  var errors = action.payload;

  var _action$meta8 = action.meta,
      csid = _action$meta8.csid,
      path = _action$meta8.path;


  return state.setIn([csid, 'validation'].concat(_toConsumableArray(path)), errors);
};

var handleValidationPassed = function handleValidationPassed(state, action) {
  var _action$meta9 = action.meta,
      csid = _action$meta9.csid,
      path = _action$meta9.path;


  return state.deleteIn([csid, 'validation'].concat(_toConsumableArray(path)));
};

var handleTransitionFulfilled = function handleTransitionFulfilled(state, action) {
  var _action$meta10 = action.meta,
      csid = _action$meta10.csid,
      transitionName = _action$meta10.transitionName;


  var updatedState = state.deleteIn([csid, 'isSavePending']);

  if (transitionName === 'delete') {
    updatedState = updatedState.delete(csid);
  }

  return updatedState;
};

exports.default = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _immutable2.default.Map();
  var action = arguments[1];

  switch (action.type) {
    case _record.VALIDATION_FAILED:
      return handleValidationFailed(state, action);
    case _record.VALIDATION_PASSED:
      return handleValidationPassed(state, action);
    case _record.ADD_FIELD_INSTANCE:
      return addFieldInstance(state, action);
    case _record.CREATE_NEW_RECORD:
      return createNewRecord(state, action);
    case _record.DELETE_FIELD_VALUE:
      return deleteFieldValue(state, action);
    case _record.MOVE_FIELD_VALUE:
      return moveFieldValue(state, action);
    case _record.SET_FIELD_VALUE:
      return setFieldValue(state, action);
    case _record.RECORD_CREATED:
      return state.set(action.meta.newRecordCsid, state.get(action.meta.currentCsid)).delete(action.meta.currentCsid);
    case _record.RECORD_READ_STARTED:
      return state.setIn([action.meta.csid, 'isReadPending'], true);
    case _record.RECORD_READ_FULFILLED:
      return handleRecordReadFulfilled(state, action);
    case _record.RECORD_READ_REJECTED:
      return state.setIn([action.meta.csid, 'error'], _immutable2.default.fromJS(action.payload)).deleteIn([action.meta.csid, 'isReadPending']);
    case _record.RECORD_SAVE_STARTED:
      return state.setIn([action.meta.csid, 'isSavePending'], true);
    case _record.RECORD_SAVE_FULFILLED:
      return handleRecordSaveFulfilled(state, action);
    case _record.RECORD_SAVE_REJECTED:
      return state.setIn([action.meta.csid, 'error'], _immutable2.default.fromJS(action.payload)).deleteIn([action.meta.csid, 'isSavePending']);
    case _record.RECORD_TRANSITION_STARTED:
      return state.setIn([action.meta.csid, 'isSavePending'], true);
    case _record.RECORD_TRANSITION_FULFILLED:
      return handleTransitionFulfilled(state, action);
    case _record.RECORD_TRANSITION_REJECTED:
      return state.deleteIn([action.meta.csid, 'isSavePending']);
    case _record.SUBRECORD_CREATED:
      return state.setIn([action.meta.csid, 'subrecord', action.meta.subrecordName], action.meta.subrecordCsid);
    case _record.SUBRECORD_READ_FULFILLED:
      return state.setIn([action.meta.csid, 'subrecord', action.meta.subrecordName], action.meta.subrecordCsid);
    case _record.REVERT_RECORD:
      return revertRecord(state, action);
    case _relation.SUBJECT_RELATIONS_UPDATED:
      return handleSubjectRelationsUpdated(state, action);
    case _idGenerator.CREATE_ID_FULFILLED:
      return handleCreateIDFulfilled(state, action);
    default:
      return state;
  }
};

var getData = exports.getData = function getData(state, csid) {
  return getCurrentData(state, csid);
};

var getError = exports.getError = function getError(state, csid) {
  return state.getIn([csid, 'error']);
};

var getSubrecordCsid = exports.getSubrecordCsid = function getSubrecordCsid(state, csid, subrecordName) {
  return state.getIn([csid, 'subrecord', subrecordName]);
};

var getRelationUpdatedTimestamp = exports.getRelationUpdatedTimestamp = function getRelationUpdatedTimestamp(state, csid) {
  return state.getIn([csid, 'relationUpdatedTime']);
};

var getNewData = exports.getNewData = function getNewData(state) {
  return getData(state, newRecordCsid);
};

var getNewSubrecordCsid = exports.getNewSubrecordCsid = function getNewSubrecordCsid(state, subrecordName) {
  return getSubrecordCsid(state, newRecordCsid, subrecordName);
};

var getValidationErrors = exports.getValidationErrors = function getValidationErrors(state, csid) {
  return state.getIn([csid, 'validation']);
};

var isReadPending = exports.isReadPending = function isReadPending(state, csid) {
  return state.getIn([csid, 'isReadPending']);
};

var isSavePending = exports.isSavePending = function isSavePending(state, csid) {
  return state.getIn([csid, 'isSavePending']);
};

var isModified = exports.isModified = function isModified(state, csid) {
  // Do a reference equality test between the current and baseline data. This will not detect if
  // a change is made, then another change is made that undoes the first. But it's more efficient
  // than a deep value equality test.

  if (getCurrentData(state, csid) !== getBaselineData(state, csid)) {
    return true;
  }

  // Check subrecords.

  var subrecords = state.getIn([csid, 'subrecord']);

  if (subrecords && subrecords.find(function (subrecordCsid) {
    return isModified(state, subrecordCsid);
  })) {
    return true;
  }

  return false;
};